/* $Workfile: OsaIoctl.h $   $Revision: 15 $    $Modtime: 4/23/08 4:04p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 1998-2003, All Rights Reserved

*******************************************************************************/

#ifndef OSA_IOCTL_H
#define OSA_IOCTL_H

#include "../Api/OsaDef.h"

// Helper functions to convert between data types for 64-bit OS.
#if defined(_WIN32_WCE)
#define OsaHandleToUlong( h ) ((unsigned long)(unsigned long *)(h) )
#define OsaPtrToUlong( p )    ((unsigned long)(unsigned long *)(p) )
#define OsaPtrToLong( p )     ((long)(long *) (p) )
#define OsaLongToPtr( l )     ((void *)(long *)((long)l))
#define OsaULongToPtr( ul )   ((void *)(unsigned long *)((unsigned long)ul))

#elif defined(OSA_WIN32) || defined(OSA_NDIS)

#define OsaHandleToUlong( h ) HandleToUlong(h)
#define OsaPtrToUlong( p )    PtrToUlong( p )
#define OsaPtrToLong( p )     PtrToLong( p )
#define OsaLongToPtr( l )     LongToPtr( l )
#define OsaULongToPtr( ul )   ULongToPtr( ul )

#elif defined(OSA_LINUX)
#define OsaHandleToUlong( h ) ((unsigned long)(unsigned long *)(h) )
#define OsaPtrToUlong( p )    ((unsigned long)(unsigned long *)(p) )
#define OsaPtrToLong( p )     ((long)(long *) (p) )
#define OsaLongToPtr( l )     ((void *)(long *)((long)l))
#define OsaULongToPtr( ul )   ((void *)(unsigned long *)((unsigned long)ul))
#endif

/*******************************************************************************
  Every OSA Ioctl struct must have one of these basic structs at the beginning.
  If not used they can be zeroed out but the space must be
  there.
*******************************************************************************/
typedef struct OsaIoctlInputS {
#if defined(_M_AMD64)
  OsaUint32  pUserBuffer;
#else
  OsaVoidP  pUserBuffer;
#endif
  OsaUint32 cbInUserBuffer;
  OsaUint32 cbUserBufferCapacity;
} OsaIoctlInputS, *OsaIoctlInputSP;


typedef struct OsaIoctlOutputS {
  OsaUint32 cbReturnedInUserBuffer;
} OsaIoctlOutputS, *OsaIoctlOutputSP;


/*******************************************************************************
*******************************************************************************/
#if defined(_WIN32_WCE)
#include <windows.h>
#include <winioctl.h>
#endif

#ifndef OSA_DRIVER
// Since this file is intended to be included in user mode as well
// we need to include the correct user mode system headers for the I/O control code handling
#if defined(WIN32)
#include <windows.h>
#include <winioctl.h>
#endif

#endif

#define OSA_DEVICE_NAME_MAX 48

#if ( defined(OSA_WIN32) || defined(WIN32) || defined(_WIN32_WCE) )
// Microsoft allows function code 2048 to 4096 for custom drivers
#define OSA_CTL_CODE(nFunctionCode) CTL_CODE(FILE_DEVICE_UNKNOWN, 2058 + nFunctionCode, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define OSA_IOCTL_DBG1 CTL_CODE(FILE_DEVICE_UNKNOWN, 2048 + 0, METHOD_BUFFERED, FILE_ANY_ACCESS)
#define OSA_IOCTL_DBG2 CTL_CODE(FILE_DEVICE_UNKNOWN, 2048 + 1, METHOD_BUFFERED, FILE_ANY_ACCESS)

#else 
// Linux (user and kernel mode and all HW platforms)
#include <linux/version.h>

#define OSA_IOC_MAGIC 'O'       // A magic value, not that important but we should choose something except 'T'

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
#define OSA_LINUX_IOWR(type,nr,size)	_IOWR(type,nr,size)
#else
/*

One of the themes of 2.6 development has been the addition  
of type checking anywhere that the compiler can be coerced  
into doing it. So the thing to do was to add checking to 
the generation of command codes. 

#define _IOC_TYPECHECK(t)               \
  ((sizeof(t) == sizeof(t[1]) &&        \
    sizeof(t) < (1 << _IOC_SIZEBITS)) ? \
    sizeof(t) : __invalid_size_argument_for_IOC)

The first test ensures that an actual type has been passed in, 
and the second makes sure it is not too large. 

*/
#define OSA_LINUX_IOWR(type,nr,size)	_IOC(_IOC_READ|_IOC_WRITE,(type),(nr),(sizeof(size)))

// PCS verify
//#define cli()			        local_irq_disable()
//#define sti()			        local_irq_enable()
//#define save_flags(x)		  local_save_flags(x)
//#define restore_flags(x)	local_irq_restore(x)
#endif

// I/O codes for the Os Extension Device
#define OSA_LINUX_OSEXT_IO_CREATE_EVENT  OSA_LINUX_IOWR(OSA_IOC_MAGIC, 0, 0)

// I/O codes for the User Event Device
#define OSA_LINUX_USREV_IO_SET_EVENT     OSA_LINUX_IOWR(OSA_IOC_MAGIC, 1, 0)

#define OSA_LINUX_OSEXT_IO_DELETE_EVENT  OSA_LINUX_IOWR(OSA_IOC_MAGIC, 2, 0)


// Defines for making externally known User Event Device names
#define OSA_USER_EVENT_NAME_PREFIX  "osaevent"  // This name + suffix digits and \0 must never exceed OSA_DEVICE_NAME_MAX

struct OsaLinuxUserEventDevice;  // Forward declaration (opaque in user mode)

// Struct for returning a User Event Device name and 
typedef struct OsaLinuxIoOutCreateEvent {
  OsaIoctlOutputS sOsaIoctlBase;
  struct OsaLinuxUserEventDevice *pEventDevice;
  char szProcEntryName[OSA_DEVICE_NAME_MAX + 1];
} OsaLinuxIoOutCreateEvent;


#define OSA_CTL_CODE(nFunctionCode)  OSA_LINUX_IOWR(OSA_IOC_MAGIC, nFunctionCode, 0)    // We pass a size of 0 (which we use to indicate a driver io code, other values indicate a code for OSA


// Linux's ioctl() only allows a buffer and no extra size arguments etc. like windows DeviceIoControl
// so we need to supply a header at the beginning of that buffer. 
typedef struct OsaLinuxIoCtlHdr {

  void *pInBuf;
  OsaUint32 cbInBufSize;

  void *pOutBuf;
  OsaUint32 cbOutBufSize;

  OsaUint32 cbReturned;

} OsaLinuxIoCtlHdr;

#endif



#endif
